(require 'mirror-url)
(require 'mirror-net)

;;; user vars
(defvar mirror-remote-copy-method 'rsync-copy-file)
(defvar mirror-rsync-program "rsync")
(defvar mirror-rcp-program "rcp")
(defvar mirror-scp-program "scp")


(defun rsync-copy-file (from to)
  (apply 'call-process
	 (list mirror-rsync-program nil (mirror-buffer) t
	       "-P"
	       from
	       to)))

(defun rcp-copy-file (from to)
  (apply 'call-process
	 (list mirror-rcp-program nil (mirror-buffer) t
	       "-p"
	       from
	       to)))

(defun scp-copy-file (from to)
  (apply 'call-process
	 (list mirror-scp-program nil (mirror-buffer) t
	       "-B"
	       from
	       to)))

(defun mirror-copy-file (from to)
  (let ((from-url (mirror-parse-url from))
	(to-url (mirror-parse-url to)))
    (if (and (murl-type from-url)
	     (murl-type to-url))
	(error "Mirror: multiple remote files: %a, %a" from to)
      (let ((mirror-type (or (murl-type from-url)
			     (murl-type to-url)))
	    (from-canonical (murl-canonical from-url))
	    (to-canonical (murl-canonical to-url)))
	(cond ((eq mirror-type 'rsync)
	       (rsync-copy-file from-canonical to-canonical))
	      ((eq mirror-type 'rcp)
	       (rcp-copy-file from-canonical to-canonical))
	      ((eq mirror-type 'scp)
	       (scp-copy-file from-canonical to-canonical))
	      (t
	       (error "Mirror: unknown URL scheme: %a" mirror-type))))
      )))


(provide 'mirror-copy)