;;; RSync/rcp/scp support for working on mirrored (as opposed
;;; to remote) directory structures

(require 'surly)
(require 'mirror-net)
(require 'mirror-url "mirror-url.el")

(defvar mirror-dir (expand-file-name "~/.xemacs-mirror")
  "working directory for mirrored directories")

(defun mfs-idem (x) x)
(defun mfs-spew (x) (message "Spew: %S" x) x)
(defun mfs-true (&rest x) t)
(defun mfs-false (&rest x) nil)


(defun mirror-local-name (murl)
  (expand-file-name
   (concat (file-name-as-directory mirror-dir)
	   (murl-host murl)
	   (murl-path murl))
   ))


(defun mirror-make-mirror-dir (murl)
  (let* ((url-obj (mirror-parse-url url))
	 (local (mirror-local-name url-obj)))
    (and (not (file-exists-p local))
	 (let ((local-dir (file-name-directory local)))
	   (and (not (file-accessible-directory-p local-dir))
		(make-directory-path local-dir)))
	 )))


(require 'mfs-io)
(require 'mfs-file-props)
(require 'mfs-file-names)
(require 'mfs-time)
(require 'mfs-security)
(require 'mfs-buffers)
(require 'mfs-file)


(add-surly "rsync://" 'mfs)
(add-surly "rcp://" 'mfs)
(add-surly "scp://" 'mfs)
